﻿using System.Diagnostics.Contracts;
using Shape.Mvp.Presenters;
using Shape.Mvp.Sample.Domain.Repositories;
using Shape.Mvp.Sample.Views;
using Shape.Mvp.Sample.Commands;
using System;

namespace Shape.Mvp.Sample.Presenters
{
   public class ExerciseListPresenter : Presenter<IExerciseListView>
   {
      private IApplicationController _applicationController;
      private IExerciseRepository _repository;
      private IExerciseListView _view;

      public ExerciseListPresenter(
         IApplicationController applicationController,
         IExerciseRepository repository, 
         IExerciseListView view) : base(view) 
      {
         Contract.Requires(applicationController != null);
         Contract.Requires(repository != null);
         Contract.Requires(view != null);

         _applicationController = applicationController;

         _repository = repository;
         _view = view;
      }

      public void Initialise()
      {
         // Attach appropriate events
         _view.ShowExerciseDetails += OnShowExerciseDetails;

         foreach (var item in _repository.GetAll())
         {
            _view.AddExercise(item.Id.ToString(), item.Name);
         }
      }

      void OnShowExerciseDetails(string id)
      {
         _applicationController.Execute<ShowExerciseDetailsCommand>(new ShowExerciseDetailsCommand(Guid.Parse(id)));
      }
   }
}
