﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Shape.Mvp.Events;
using Shape.Mvp.Commands;
using System.Diagnostics.Contracts;
using Shape.Mvp.Views;
using System.Windows.Forms;

namespace Shape.Mvp
{
   public class DefaultApplicationController : IApplicationController
   {
      private IEventAggregator _eventAggregator;
      private ICommandHandlerFactory _commandHandlerFactory;

      public DefaultApplicationController(
         IEventAggregator eventAggregator, 
         ICommandHandlerFactory commandHandlerFactory) 
      { 
         Contract.Requires(eventAggregator != null);
         Contract.Requires(commandHandlerFactory != null);

         _eventAggregator = eventAggregator;
         _commandHandlerFactory = commandHandlerFactory;
      }

      public void Execute<TCommand>(TCommand command) where TCommand : Command
      {
         var commandHandler = _commandHandlerFactory.CreateCommandHandler<TCommand>();

         if (commandHandler != null)
         {
            commandHandler.Execute(command);
         }
      }

      public void Publish<TEvent>(TEvent item)
      {
         _eventAggregator.Publish<TEvent>(item);
      }

      public void AddEventSubscriber(ISubscriber subscriber)
      {
         _eventAggregator.Subscribe(subscriber);
      }

		private WindowManager _windowManager = new WindowManager();

      public IWindowMode Show(IView view)
      {
         return _windowManager.Show(view);
      }

   }
}
